@@ -1,58 +1,52 @@
+"""
+Tests the LDAP C Extension module called _ldap
+"""
 
+import os
 import unittest
+from slapdtest import SlapdTestCase
+
+# Switch off processing .ldaprc or ldap.conf before importing _ldap
+os.environ['LDAPNOINIT'] = '1'
+
+# import the plain C wrapper module
 import _ldap
-import logging
 
-from Tests import slapd
 
-reusable_server = None
-def get_reusable_server():
-    global reusable_server
-    if reusable_server is None:
-        reusable_server = slapd.Slapd()
-    return reusable_server
+class TestLdapCExtension(SlapdTestCase):
+    """
+    These tests apply only to the _ldap module and therefore bypass the
+    LDAPObject wrapper completely.
+    """
 
-class TestLdapCExtension(unittest.TestCase):
-    """Tests the LDAP C Extension module, _ldap.
-       These tests apply only to the _ldap module and bypass the
-       LDAPObject wrapper completely."""
+    timeout = 5
 
-    timeout = 3
-
-    def _init_server(self, reuse_existing=True):
-        global reusable_server
-        """Sets self.server to a test LDAP server and self.base
-           to its base"""
-        if reuse_existing:
-            server = get_reusable_server()
-        else:
-            server = slapd.Slapd() # private server
-        #server.set_debug()  # enables verbose messages
-        server.start()   # no effect if already started
-        self.server = server
-        self.base = server.get_dn_suffix()
-        return server
-
-    def _init(self, reuse_existing=True, bind=True):
-        """Starts a server, and returns a LDAPObject bound to it"""
-        server = self._init_server(reuse_existing)
-        l = _ldap.initialize(server.get_url())
+    def _open_conn(self, bind=True):
+        """
+        Starts a server, and returns a LDAPObject bound to it
+        """
+        l = _ldap.initialize(self.server.ldap_uri)
         if bind:
             # Perform a simple bind
             l.set_option(_ldap.OPT_PROTOCOL_VERSION, _ldap.VERSION3)
-            m = l.simple_bind(server.get_root_dn(), server.get_root_password())
-            result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ONE, self.timeout)
-            self.assertTrue(result, _ldap.RES_BIND)
+            m = l.simple_bind(self.server.root_dn, self.server.root_pw)
+            result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ONE, self.timeout)
+            self.assertEqual(result, _ldap.RES_BIND)
+            self.assertEqual(type(msgid), type(0))
         return l
 
     def assertNotNone(self, expr, msg=None):
         self.failIf(expr is None, msg or repr(expr))
+
     def assertNone(self, expr, msg=None):
         self.failIf(expr is not None, msg or repr(expr))
 
     # Test for the existence of a whole bunch of constants
     # that the C module is supposed to export
     def test_constants(self):
+        """
+        Test whether all libldap-derived constants are correct
+        """
         self.assertEquals(_ldap.PORT, 389)
         self.assertEquals(_ldap.VERSION1, 1)
         self.assertEquals(_ldap.VERSION2, 2)
@@ -146,7 +140,7 @@ class TestLdapCExtension(unittest.TestCase):
         self.assertNotNone(_ldap.AVA_NONPRINTABLE)
 
         # these two constants are pointless? XXX
-        self.assertEquals(_ldap.OPT_ON, 1) 
+        self.assertEquals(_ldap.OPT_ON, 1)
         self.assertEquals(_ldap.OPT_OFF, 0)
 
         # these constants useless after ldap_url_parse() was dropped XXX
@@ -154,12 +148,13 @@ class TestLdapCExtension(unittest.TestCase):
         self.assertNotNone(_ldap.URL_ERR_MEM)
 
     def test_simple_bind(self):
-        l = self._init()
+        l = self._open_conn()
 
     def test_simple_anonymous_bind(self):
-        l = self._init(bind=False)
+        l = self._open_conn(bind=False)
         m = l.simple_bind("", "")
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        self.assertEqual(type(m), type(0))
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertTrue(result, _ldap.RES_BIND)
         self.assertEquals(msgid, m)
         self.assertEquals(pmsg, [])
@@ -167,66 +162,67 @@ class TestLdapCExtension(unittest.TestCase):
 
         # see if we can get the rootdse while we're here
         m = l.search_ext("", _ldap.SCOPE_BASE, '(objectClass=*)')
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_SEARCH_RESULT)
         self.assertEquals(pmsg[0][0], "") # rootDSE has no dn
         self.assertEquals(msgid, m)
         self.assertTrue(pmsg[0][1].has_key('objectClass'))
 
     def test_unbind(self):
-        l = self._init()
+        l = self._open_conn()
         m = l.unbind_ext()
         self.assertNone(m)
-
         # Second attempt to unbind should yield an exception
-        try: l.unbind_ext()
-        except _ldap.error: pass
+        try:
+            l.unbind_ext()
+        except _ldap.error:
+            pass
 
     def test_search_ext_individual(self):
-        l = self._init()
-
-        m = l.search_ext(self.base, _ldap.SCOPE_SUBTREE, 
-                '(objectClass=dcObject)')
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ONE, self.timeout)
-
+        l = self._open_conn()
+        # send search request
+        m = l.search_ext(
+            self.server.suffix,
+            _ldap.SCOPE_SUBTREE,
+            '(objectClass=dcObject)'
+        )
+        self.assertEqual(type(m), type(0))
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ONE, self.timeout)
         # Expect to get just one object
         self.assertEquals(result, _ldap.RES_SEARCH_ENTRY)
         self.assertEquals(len(pmsg), 1)
         self.assertEquals(len(pmsg[0]), 2)
-        self.assertEquals(pmsg[0][0], self.base)
-        self.assertEquals(pmsg[0][0], self.base)
+        self.assertEquals(pmsg[0][0], self.server.suffix)
+        self.assertEquals(pmsg[0][0], self.server.suffix)
         self.assertTrue('dcObject' in pmsg[0][1]['objectClass'])
         self.assertTrue('organization' in pmsg[0][1]['objectClass'])
         self.assertEquals(msgid, m)
         self.assertEquals(ctrls, [])
 
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ONE, self.timeout)
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ONE, self.timeout)
         self.assertEquals(result, _ldap.RES_SEARCH_RESULT)
         self.assertEquals(pmsg, [])
         self.assertEquals(msgid, m)
         self.assertEquals(ctrls, [])
 
     def test_abandon(self):
-        l = self._init()
-
-        m = l.search_ext(self.base, _ldap.SCOPE_SUBTREE, '(objectClass=*)')
-
+        l = self._open_conn()
+        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(objectClass=*)')
         ret = l.abandon_ext(m)
         self.assertNone(ret)
-
-        got_timeout = False
         try:
             r = l.result4(m, _ldap.MSG_ALL, 0.3)  # (timeout /could/ be longer)
         except _ldap.TIMEOUT, e:
-            got_timeout = True
-        self.assertTrue(got_timeout)
+            pass
+        else:
+            self.fail("expected TIMEOUT, got %r" % r)
 
     def test_search_ext_all(self):
-        l = self._init()
-
-        m = l.search_ext(self.base, _ldap.SCOPE_SUBTREE, '(objectClass=*)')
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
-
+        l = self._open_conn()
+        # send search request
+        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(objectClass=*)')
+        self.assertEqual(type(m), type(0))
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         # Expect to get some objects
         self.assertEquals(result, _ldap.RES_SEARCH_RESULT)
         self.assertTrue(len(pmsg) >= 2)
@@ -234,204 +230,237 @@ class TestLdapCExtension(unittest.TestCase):
         self.assertEquals(ctrls, [])
 
     def test_add(self):
-        l = self._init()
-
-        m = l.add_ext("cn=Foo," + self.base, [
-               ('objectClass','organizationalRole'), 
-               ('cn', 'Foo'),
-               ('description', 'testing'),
-            ])
-
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        """
+        test add operation
+        """
+        l = self._open_conn()
+        m = l.add_ext(
+            "cn=Foo," + self.server.suffix,
+            [
+                ('objectClass', 'organizationalRole'),
+                ('cn', 'Foo'),
+                ('description', 'testing'),
+            ]
+        )
+        self.assertEqual(type(m), type(0))
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_ADD)
         self.assertEquals(pmsg, [])
         self.assertEquals(msgid, m)
         self.assertEquals(ctrls, [])
-
         # search for it back
-        m = l.search_ext(self.base, _ldap.SCOPE_SUBTREE, '(cn=Foo)')
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
-
+        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(cn=Foo)')
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         # Expect to get the objects
         self.assertEquals(result, _ldap.RES_SEARCH_RESULT)
         self.assertEquals(len(pmsg), 1)
         self.assertEquals(msgid, m)
         self.assertEquals(ctrls, [])
-
-        self.assertEquals(pmsg[0], ('cn=Foo,'+self.base,
-                { 'objectClass': ['organizationalRole'],
-                  'cn': ['Foo'],
-                  'description': ['testing'] }))
+        self.assertEquals(
+            pmsg[0],
+            (
+                'cn=Foo,'+self.server.suffix,
+                {
+                    'objectClass': ['organizationalRole'],
+                    'cn': ['Foo'],
+                    'description': ['testing'],
+                }
+            )
+        )
 
     def test_compare(self):
-        l = self._init()
-
+        """
+        test compare operation
+        """
+        l = self._open_conn()
         # first, add an object with a field we can compare on
-        dn = "cn=CompareTest," + self.base
-        m = l.add_ext(dn, [
-               ('objectClass','person'), 
-               ('sn', 'CompareTest'),
-               ('cn', 'CompareTest'),
-               ('userPassword', 'the_password'),
-            ])
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        dn = "cn=CompareTest," + self.server.suffix
+        m = l.add_ext(
+            dn,
+            [
+                ('objectClass', 'person'),
+                ('sn', 'CompareTest'),
+                ('cn', 'CompareTest'),
+                ('userPassword', 'the_password'),
+            ],
+        )
+        self.assertEqual(type(m), type(0))
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_ADD)
-
         # try a false compare
         m = l.compare_ext(dn, "userPassword", "bad_string")
-        compared_false = False
         try:
             r = l.result4(m, _ldap.MSG_ALL, self.timeout)
-            self.fail(repr(r))
         except _ldap.COMPARE_FALSE:
-            compared_false = True
-        self.assertTrue(compared_false)
-
+            pass
+        else:
+            self.fail("expected COMPARE_FALSE, got %r" % r)
         # try a true compare
         m = l.compare_ext(dn, "userPassword", "the_password")
-        compared_true = False
         try:
             r = l.result4(m, _ldap.MSG_ALL, self.timeout)
-            self.fail(repr(r))
         except _ldap.COMPARE_TRUE:
-            compared_true = True
-        self.assertTrue(compared_true)
-
+            pass
+        else:
+            self.fail("expected COMPARE_TRUE, got %r" % r)
+        # try a compare on bad attribute
         m = l.compare_ext(dn, "badAttribute", "ignoreme")
-        raised_error = False
         try:
             r = l.result4(m, _ldap.MSG_ALL, self.timeout)
-            self.fail(repr(r))
         except _ldap.error:
-            raised_error = True
-        self.assertTrue(raised_error)
+            pass
+        else:
+            self.fail("expected LDAPError, got %r" % r)
 
     def test_delete_no_such_object(self):
-        l = self._init()
-
-        # try deleting an object that doesn't exist
-        not_found = False
-        m = l.delete_ext("cn=DoesNotExist,"+self.base)
+        """
+        try deleting an object that doesn't exist
+        """
+        l = self._open_conn()
+        m = l.delete_ext("cn=DoesNotExist,"+self.server.suffix)
         try:
             r = l.result4(m, _ldap.MSG_ALL, self.timeout)
-            self.fail(r)
         except _ldap.NO_SUCH_OBJECT:
-            not_found = True
-        self.assertTrue(not_found)
+            pass
+        else:
+            self.fail("expected NO_SUCH_OBJECT, got %r" % r)
 
     def test_delete(self):
-        l = self._init()
+        l = self._open_conn()
         # first, add an object we will delete
-        dn = "cn=Deleteme,"+self.base
-        m = l.add_ext(dn, [
-               ('objectClass','organizationalRole'), 
-               ('cn', 'Deleteme'),
-            ])
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        dn = "cn=Deleteme,"+self.server.suffix
+        m = l.add_ext(
+            dn,
+            [
+                ('objectClass', 'organizationalRole'),
+                ('cn', 'Deleteme'),
+            ]
+        )
+        self.assertEqual(type(m), type(0))
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_ADD)
 
         m = l.delete_ext(dn)
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_DELETE)
         self.assertEquals(msgid, m)
         self.assertEquals(pmsg, [])
         self.assertEquals(ctrls, [])
 
     def test_modify_no_such_object(self):
-        l = self._init()
+        l = self._open_conn()
 
         # try deleting an object that doesn't exist
-        not_found = False
-        m = l.modify_ext("cn=DoesNotExist,"+self.base, [
+        m = l.modify_ext(
+            "cn=DoesNotExist,"+self.server.suffix,
+            [
                 (_ldap.MOD_ADD, 'description', ['blah']),
-            ])
+            ]
+        )
         try:
             r = l.result4(m, _ldap.MSG_ALL, self.timeout)
-            self.fail(r)
         except _ldap.NO_SUCH_OBJECT:
-            not_found = True
-        self.assertTrue(not_found)
+            pass
+        else:
+            self.fail("expected NO_SUCH_OBJECT, got %r" % r)
 
-    def DISABLED_test_modify_no_such_object_empty_attrs(self):
-        # XXX ldif-backend for slapd appears broken???
-
-        l = self._init()
-
-        # try deleting an object that doesn't exist
-        m = l.modify_ext("cn=DoesNotExist,"+self.base, [
-                (_ldap.MOD_ADD, 'description', []),
-            ])
+    def test_modify_no_such_object_empty_attrs(self):
+        """
+        try deleting an object that doesn't exist
+        """
+        l = self._open_conn()
+        m = l.modify_ext(
+            "cn=DoesNotExist,"+self.server.suffix,
+            [
+                (_ldap.MOD_ADD, 'description', ['dummy']),
+            ]
+        )
         self.assertTrue(isinstance(m, int))
-        r = l.result4(m, _ldap.MSG_ALL, self.timeout) # what should happen??
-        self.fail(r)
+        try:
+            r = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        except _ldap.NO_SUCH_OBJECT:
+            pass
+        else:
+            self.fail("expected NO_SUCH_OBJECT, got %r" % r)
 
     def test_modify(self):
-        l = self._init()
+        """
+        test modify operation
+        """
+        l = self._open_conn()
         # first, add an object we will delete
-        dn = "cn=AddToMe,"+self.base
-        m = l.add_ext(dn, [
-               ('objectClass','person'), 
-               ('cn', 'AddToMe'),
-               ('sn', 'Modify'),
-               ('description', 'a description'),
-            ])
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        dn = "cn=AddToMe,"+self.server.suffix
+        m = l.add_ext(
+            dn,
+            [
+                ('objectClass', 'person'),
+                ('cn', 'AddToMe'),
+                ('sn', 'Modify'),
+                ('description', 'a description'),
+            ]
+        )
+        self.assertEqual(type(m), type(0))
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_ADD)
 
-        m = l.modify_ext(dn, [
+        m = l.modify_ext(
+            dn,
+            [
                 (_ldap.MOD_ADD, 'description', ['b desc', 'c desc']),
-            ])
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+            ]
+        )
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_MODIFY)
         self.assertEquals(pmsg, [])
         self.assertEquals(msgid, m)
         self.assertEquals(ctrls, [])
-
         # search for it back
-        m = l.search_ext(self.base, _ldap.SCOPE_SUBTREE, '(cn=AddToMe)')
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
-
+        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(cn=AddToMe)')
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         # Expect to get the objects
         self.assertEquals(result, _ldap.RES_SEARCH_RESULT)
         self.assertEquals(len(pmsg), 1)
         self.assertEquals(msgid, m)
         self.assertEquals(ctrls, [])
-
         self.assertEquals(pmsg[0][0], dn)
         d = list(pmsg[0][1]['description'])
         d.sort()
         self.assertEquals(d, ['a description', 'b desc', 'c desc'])
 
     def test_rename(self):
-        l = self._init()
-        dn = "cn=RenameMe,"+self.base
-        m = l.add_ext(dn, [
-               ('objectClass','organizationalRole'), 
-               ('cn', 'RenameMe'),
-            ])
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        l = self._open_conn()
+        dn = "cn=RenameMe,"+self.server.suffix
+        m = l.add_ext(
+            dn,
+            [
+                ('objectClass', 'organizationalRole'),
+                ('cn', 'RenameMe'),
+            ]
+        )
+        self.assertEqual(type(m), type(0))
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_ADD)
 
         # do the rename with same parent
         m = l.rename(dn, "cn=IAmRenamed")
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_MODRDN)
         self.assertEquals(msgid, m)
         self.assertEquals(pmsg, [])
         self.assertEquals(ctrls, [])
 
         # make sure the old one is gone
-        m = l.search_ext(self.base, _ldap.SCOPE_SUBTREE, '(cn=RenameMe)')
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(cn=RenameMe)')
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_SEARCH_RESULT)
         self.assertEquals(len(pmsg), 0) # expect no results
         self.assertEquals(msgid, m)
         self.assertEquals(ctrls, [])
 
         # check that the new one looks right
-        dn2 = "cn=IAmRenamed,"+self.base
-        m = l.search_ext(self.base, _ldap.SCOPE_SUBTREE, '(cn=IAmRenamed)')
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        dn2 = "cn=IAmRenamed,"+self.server.suffix
+        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(cn=IAmRenamed)')
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_SEARCH_RESULT)
         self.assertEquals(msgid, m)
         self.assertEquals(ctrls, [])
@@ -440,23 +469,29 @@ class TestLdapCExtension(unittest.TestCase):
         self.assertEquals(pmsg[0][1]['cn'], ['IAmRenamed'])
 
         # create the container
-        containerDn = "ou=RenameContainer,"+self.base
-        m = l.add_ext(containerDn, [
-               ('objectClass','organizationalUnit'), 
-               ('ou', 'RenameContainer'),
-            ])
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        containerDn = "ou=RenameContainer,"+self.server.suffix
+        m = l.add_ext(
+            containerDn,
+            [
+                ('objectClass', 'organizationalUnit'),
+                ('ou', 'RenameContainer'),
+            ]
+        )
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_ADD)
 
-        # WORKAROUND bug in slapd. (Without an existing child, 
+        # WORKAROUND bug in slapd. (Without an existing child,
         # renames into a container object do not work for the ldif backend,
         # the renamed object appears to be deleted, not moved.)
         # see http://www.openldap.org/its/index.cgi/Software%20Bugs?id=5408
-        m = l.add_ext("cn=Bogus," + containerDn, [
-               ('objectClass','organizationalRole'), 
-               ('cn', 'Bogus'),
-            ])
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        m = l.add_ext(
+            "cn=Bogus," + containerDn,
+            [
+                ('objectClass', 'organizationalRole'),
+                ('cn', 'Bogus'),
+            ]
+        )
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_ADD)
 
         # now rename from dn2 to the conater
@@ -464,26 +499,26 @@ class TestLdapCExtension(unittest.TestCase):
 
         # Now try renaming dn2 across container (simultaneous name change)
         m = l.rename(dn2, "cn=IAmRenamedAgain", containerDn)
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_MODRDN)
         self.assertEquals(msgid, m)
         self.assertEquals(pmsg, [])
         self.assertEquals(ctrls, [])
 
         # make sure dn2 is gone
-        m = l.search_ext(self.base, _ldap.SCOPE_SUBTREE, '(cn=IAmRenamed)')
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(cn=IAmRenamed)')
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_SEARCH_RESULT)
         self.assertEquals(len(pmsg), 0) # expect no results
         self.assertEquals(msgid, m)
         self.assertEquals(ctrls, [])
 
-        m = l.search_ext(self.base, _ldap.SCOPE_SUBTREE, '(objectClass=*)')
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(objectClass=*)')
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
 
         # make sure dn3 is there
-        m = l.search_ext(self.base, _ldap.SCOPE_SUBTREE, '(cn=IAmRenamedAgain)')
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        m = l.search_ext(self.server.suffix, _ldap.SCOPE_SUBTREE, '(cn=IAmRenamedAgain)')
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_SEARCH_RESULT)
         self.assertEquals(msgid, m)
         self.assertEquals(ctrls, [])
@@ -493,53 +528,55 @@ class TestLdapCExtension(unittest.TestCase):
 
 
     def test_whoami(self):
-        l = self._init()
+        l = self._open_conn()
         r = l.whoami_s()
-        self.assertEquals("dn:" + self.server.get_root_dn(), r)
+        self.assertEquals("dn:" + self.server.root_dn, r)
 
     def test_whoami_unbound(self):
-        l = self._init(bind=False)
+        l = self._open_conn(bind=False)
         l.set_option(_ldap.OPT_PROTOCOL_VERSION, _ldap.VERSION3)
         r = l.whoami_s()
         self.assertEquals("", r)
 
     def test_whoami_anonymous(self):
-        l = self._init(bind=False)
+        l = self._open_conn(bind=False)
         l.set_option(_ldap.OPT_PROTOCOL_VERSION, _ldap.VERSION3)
-
         # Anonymous bind
         m = l.simple_bind("", "")
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertTrue(result, _ldap.RES_BIND)
-
+        # check with Who Am I? extended operation
         r = l.whoami_s()
         self.assertEquals("", r)
 
     def test_passwd(self):
-        l = self._init()
-
+        l = self._open_conn()
         # first, create a user to change password on
-        dn = "cn=PasswordTest," + self.base
-        m = l.add_ext(dn, [
-               ('objectClass','person'), 
-               ('sn', 'PasswordTest'),
-               ('cn', 'PasswordTest'),
-               ('userPassword', 'initial'),
-            ])
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        dn = "cn=PasswordTest," + self.server.suffix
+        m = l.add_ext(
+            dn,
+            [
+                ('objectClass', 'person'),
+                ('sn', 'PasswordTest'),
+                ('cn', 'PasswordTest'),
+                ('userPassword', 'initial'),
+            ]
+        )
+        self.assertEqual(type(m), type(0))
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(result, _ldap.RES_ADD)
-
         # try changing password with a wrong old-pw
         m = l.passwd(dn, "bogus", "ignored")
+        self.assertEqual(type(m), type(0))
         try:
             r = l.result4(m, _ldap.MSG_ALL, self.timeout)
-            self.fail("expected UNWILLING_TO_PERFORM")
         except _ldap.UNWILLING_TO_PERFORM:
             pass
-
+        else:
+            self.fail("expected UNWILLING_TO_PERFORM, got %r" % r)
         # try changing password with a correct old-pw
         m = l.passwd(dn, "initial", "changed")
-        result,pmsg,msgid,ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
+        result, pmsg, msgid, ctrls = l.result4(m, _ldap.MSG_ALL, self.timeout)
         self.assertEquals(msgid, m)
         self.assertEquals(pmsg, [])
         self.assertEquals(result, _ldap.RES_EXTENDED)
@@ -551,8 +588,10 @@ class TestLdapCExtension(unittest.TestCase):
 
             try:
                 _ldap.set_option(_ldap.OPT_PROTOCOL_VERSION, "3")
-                self.fail("expected string value to raise a type error")
-            except TypeError: pass
+            except TypeError:
+                pass
+            else:
+                self.fail("expected string value to raise a TypeError")
 
             _ldap.set_option(_ldap.OPT_PROTOCOL_VERSION, _ldap.VERSION2)
             v = _ldap.get_option(_ldap.OPT_PROTOCOL_VERSION)
@@ -563,7 +602,7 @@ class TestLdapCExtension(unittest.TestCase):
         finally:
             _ldap.set_option(_ldap.OPT_PROTOCOL_VERSION, oldval)
 
-        l = self._init()
+        l = self._open_conn()
 
         # Try changing some basic options and checking that they took effect
 
@@ -578,9 +617,10 @@ class TestLdapCExtension(unittest.TestCase):
         # Try setting options that will yield a known error.
         try:
             _ldap.get_option(_ldap.OPT_MATCHED_DN)
-            self.fail("expected ValueError")
         except ValueError:
             pass
+        else:
+            self.fail("expected ValueError")
 
     def _require_attr(self, obj, attrname):
         """Returns true if the attribute exists on the object.
@@ -595,24 +635,22 @@ class TestLdapCExtension(unittest.TestCase):
         return False
 
     def test_sasl(self):
-        l = self._init()
+        l = self._open_conn()
         if not self._require_attr(l, 'sasl_interactive_bind_s'): # HAVE_SASL
             return
         # TODO
 
     def test_tls(self):
-        l = self._init()
+        l = self._open_conn()
         if not self._require_attr(l, 'start_tls_s'):    # HAVE_TLS
             return
         # TODO
 
     def test_cancel(self):
-        l = self._init()
+        l = self._open_conn()
         if not self._require_attr(l, 'cancel'):         # FEATURE_CANCEL
             return
 
-    def test_str2dn(self):
-        pass
 
 if __name__ == '__main__':
     unittest.main()
